using System.Reactive.Linq;
using System.Reactive.Subjects;
using MediatR;
using Recyclarr.Notifications.Events;

namespace Recyclarr.Notifications;

public class NotificationEmitter(IMediator mediator)
{
    private readonly Subject<IPresentableNotification> _notifications = new();

    public IObservable<IPresentableNotification> OnNotification => _notifications.AsObservable();

    public void NotifyStatistic<T>(string description, T stat) where T : notnull
    {
        mediator.Send(new StatisticEvent(description, stat.ToString() ?? "Stat Error"));
    }

    public void NotifyError(string error)
    {
        _notifications.OnNext(new ErrorEvent(error));
    }

    public void WarningIssued()
    {
        // _notifications.OnNext(new WarningEvent());
    }
}
