using Autofac;
using MediatR;
using Recyclarr.Notifications.Apprise;

namespace Recyclarr.Notifications;

public class NotificationsAutofacModule : Module
{
    protected override void Load(ContainerBuilder builder)
    {
        base.Load(builder);

        builder.RegisterAssemblyTypes(ThisAssembly)
            .AsClosedTypesOf(typeof(IRequestHandler<>))
            .AsImplementedInterfaces()
            .InstancePerMatchingLifetimeScope(NotificationScope.ScopeName);

        builder.RegisterType<NotificationService>().InstancePerMatchingLifetimeScope(NotificationScope.ScopeName);
        builder.RegisterType<NotificationEmitter>().InstancePerMatchingLifetimeScope(NotificationScope.ScopeName);

        // Apprise
        builder.RegisterType<AppriseNotificationApiService>().As<IAppriseNotificationApiService>();
        builder.RegisterType<AppriseRequestBuilder>().As<IAppriseRequestBuilder>();
    }
}
